@@ -1,11 +1,10 @@
 //
 //  main_functions.cpp
-//  
+//
 //
 //  Created by Thijs Janzen on 07/11/2017.
 //
 //
-
 #include <stdio.h>
 #include "Output.h"
 #include "Fish.h"
@@ -14,21 +13,18 @@
 #include <Rcpp.h>
 using namespace Rcpp;
 
-
-
 Output doSimulation_inf(int popSize,
                     double initRatio,
                     int maxTime,
                     double numRecombinations,
-                    int numberOfMarkers
-                    )    {
-
+                    int numberOfMarkers,
+                    rnd_t& rndgen)    {
     Output O;
     std::vector<Fish_inf> Pop;
     std::vector<double> markers;
     if(numberOfMarkers > 0) {
         for(int i = 0; i < numberOfMarkers; ) {
-            double pos = uniform();
+            double pos = rndgen.uniform();
             if(pos > 0 && pos < 1.0) {
                 ++i;
                 markers.push_back(pos);
@@ -37,28 +33,25 @@ Output doSimulation_inf(int popSize,
         std::sort(markers.begin(), markers.end());
     }
 
+    O.markers = markers;
 
     Fish_inf parent1 = Fish_inf(0);
     Fish_inf parent2 = Fish_inf(1);
 
     for(int i = 0; i < popSize; ++i) {
-        Fish_inf p1 = parent1;
-        Fish_inf p2 = parent1;
+        Fish_inf p1 = parent2;
+        Fish_inf p2 = parent2;
 
-        if(uniform() < initRatio) {
-            p1 = parent2;
+        if(rndgen.uniform() < initRatio) {
+            p1 = parent1;
         }
-        if(uniform() < initRatio) {
-            p2 = parent2;
+        if(rndgen.uniform() < initRatio) {
+            p2 = parent1;
         }
 
-
-        Pop.push_back(mate_inf(p1,p2, numRecombinations));
+        Pop.push_back(mate_inf(p1,p2, numRecombinations, rndgen));
     }
 
-  //  Rcout << "0--------25--------50--------75--------100\n";
-  //  Rcout << "*";
-
     int updateFreq = maxTime / 20;
     if(updateFreq < 1) updateFreq = 1;
 
@@ -66,25 +59,21 @@ Output doSimulation_inf(int popSize,
         O.update_inf(Pop);
         if(numberOfMarkers > 0) O.detectNumJunctions(Pop, markers);
 
-        std::vector<Fish_inf> newGeneration;
+        std::vector<Fish_inf> newGeneration(popSize);
 
         for(int i = 0; i < popSize; ++i)  {
-            int index1 = random_number(popSize);
-            int index2 = random_number(popSize);
+            int index1 = rndgen.random_number(popSize);
+            int index2 = rndgen.random_number(popSize);
 
-            Fish_inf kid = mate_inf(Pop[index1], Pop[index2], numRecombinations);
-
-            newGeneration.push_back(kid);
+            newGeneration[i] = mate_inf(Pop[index1],
+                                        Pop[index2],
+                                           numRecombinations,
+                                           rndgen);
         }
-        
+
         Pop = newGeneration;
         newGeneration.clear();
-
-        if(t % updateFreq == 0) {
-      //      Rcout << "**";
-        }
     }
-    //Rcout << "\n";
     return O;
 }
 
@@ -92,8 +81,8 @@ Output doSimulation_fin(int popSize,
                         int genomeSize,
                         double initRatio,
                         int maxTime,
-                        double numRecombinations
-                    )    {
+                        double numRecombinations,
+                        rnd_t& rndgen)    {
     Output O;
     std::vector<Fish_fin> Pop;
 
@@ -101,98 +90,81 @@ Output doSimulation_fin(int popSize,
     Fish_fin parent2 = Fish_fin(1, genomeSize);
 
     for(int i = 0; i < popSize; ++i) {
-        Fish_fin p1 = parent1;
-        Fish_fin p2 = parent1;
+        Fish_fin p1 = parent2;
+        Fish_fin p2 = parent2;
 
-        if(uniform() < initRatio) {
-            p1 = parent2;
+        if(rndgen.uniform() < initRatio) {
+            p1 = parent1;
         }
-        if(uniform() < initRatio) {
-            p2 = parent2;
+        if(rndgen.uniform() < initRatio) {
+            p2 = parent1;
         }
 
-        Pop.push_back(mate_fin(p1,p2, numRecombinations));
+        Pop.push_back(mate_fin(p1,p2, numRecombinations, rndgen));
     }
 
-  //  Rcout << "0--------25--------50--------75--------100\n";
-  //  Rcout << "*";
-
     int updateFreq = maxTime / 20;
     if(updateFreq < 1) updateFreq = 1;
 
-    for(int t = 0; t < maxTime; ++t) {
+    for(int t = 0; t <= maxTime; ++t) {
         O.update_fin(Pop);
-        std::vector<Fish_fin> newGeneration;
+        std::vector<Fish_fin> newGeneration(popSize);
 
         for(int i = 0; i < popSize; ++i)
         {
-            int index1 = random_number(popSize);
-            int index2 = random_number(popSize);
+            int index1 = rndgen.random_number(popSize);
+            int index2 = rndgen.random_number(popSize);
 
-            Fish_fin kid;
-            kid = mate_fin(Pop[index1],Pop[index2], numRecombinations);
-
-            newGeneration.push_back(kid);
+            newGeneration[i] = mate_fin(Pop[index1],
+                                        Pop[index2],
+                                           numRecombinations,
+                                           rndgen);
         }
 
         Pop = newGeneration;
         newGeneration.clear();
-
-        if(t % updateFreq == 0) {
-    //        Rcout << "**";
-        }
     }
-  //  Rcout << "\n";
+
     return O;
 }
 
 // [[Rcpp::export]]
 List sim_fin_chrom(int pop_size,
-                   double init_heterozygosity,
+                   double freq_ancestor_1,
                    int run_time,
                    double size_in_Morgan,
                    int seed,
                    int R) {
 
-    double p = 0.5 * (1 - sqrt(1 - 2 * init_heterozygosity));
+  rnd_t rndgen(seed);
 
-    //Rcout << "sim_fin_chrom, let's go!\n";
+  Output O = doSimulation_fin(pop_size,
+                              R + 1,
+                              freq_ancestor_1,
+                              run_time,
+                              size_in_Morgan,
+                              rndgen);
 
-    Output O = doSimulation_fin(pop_size,
-                                R + 1,
-                                p,
-                                run_time,
-                                size_in_Morgan);
-    
-    return List::create(Named("avgJunctions") = O.avgJunctions);
+  return List::create(Named("avgJunctions") = O.avgJunctions);
 }
 
 // [[Rcpp::export]]
 List sim_inf_chrom(int pop_size,
-                   double init_heterozygosity,
+                   double freq_ancestor_1,
                    int run_time,
                    double size_in_Morgan,
                    int markers,
                    int seed) {
-    double p = 0.5 * (1 - sqrt(1 - 2 * init_heterozygosity));
-
-    //Rcout << "sim_inf_chrom, let's go!\n";
+  rnd_t rndgen(seed);
 
     Output O = doSimulation_inf(pop_size,
-                                p,
+                                freq_ancestor_1,
                                 run_time,
                                 size_in_Morgan,
-                                markers);
+                                markers,
+                                rndgen);
+
     return List::create(Named("avgJunctions") = O.avgJunctions,
-                        Named("detectedJunctions") = O.avg_detected_Junctions);
+                        Named("detectedJunctions") = O.avg_detected_Junctions,
+                        Named("markers") = O.markers);
 }
-
-
-
-
-
-
-
-
-
-
